@@ -1,19 +1,44 @@
 # -*- coding: utf-8 -*-
+'''
+>>> s = SubRipFile("""1
+... 00:00:02,207 --> 00:00:03,221
+... ÉCHEC !
+...
+... 2
+... 00:00:30,371 --> 00:00:33,981
+... Le Retour de l'Enfant Prodigue
+...
+... 3
+... 00:00:35,619 --> 00:00:37,277
+... Ce qui veut dire...
+...
+... 4
+... 00:00:37,433 --> 00:00:39,761
+... un bénéfice, cette année, de...
+...
+... 5
+... 00:00:39,933 --> 00:00:43,121
+... - 1 800 milliards de milliards.
+... - Splendide !
+...
+... 6
+... 00:00:43,410 --> 00:00:45,136
+... Sans compter les revenus subsidiaires
+... """)
+>>> len(s)
+6
+'''
 __all__ = ['SubRipFile', 'SubRipItem', 'SubRipTime']
 import re
-import sys
-from os import path
+from itertools import chain
+from os.path import exists, isfile
 from UserList import UserList
 try:
-    from cStringIO import cStringIO as StringIO
+    from cStringIO import StringIO
 except ImportError:
     from StringIO import StringIO
 
 
-def str_keys(dic):
-    return dict((str(k), v) for k, v in dic.iteritems())
-
-
 class InvalidItem(Exception):
     pass
 
@@ -56,7 +81,8 @@ class SubRipTime(object):
     micro_seconds = TimeItemDescriptor(1, SECONDS_RATIO)
 
     def __init__(self, hours=0, minutes=0, seconds=0, micro_seconds=0):
-        """SubRipTime(hours, minutes, seconds, micro_seconds)
+        """
+        SubRipTime(hours, minutes, seconds, micro_seconds)
 
         All arguments are optional and have a default value of 0.
         """
@@ -70,14 +96,15 @@ class SubRipTime(object):
             self.minutes, self.seconds, self.micro_seconds)
 
     def shift(self, *args, **kwargs):
-        """shift(hours, minutes, seconds, micro_seconds)
+        """
+        shift(hours, minutes, seconds, micro_seconds)
 
         All arguments are optional and have a default value of 0.
         """
         self.ordinal += self.__class__(*args, **kwargs).ordinal
 
     @classmethod
-    def from_ordinal(self, ordinal):
+    def from_ordinal(cls, ordinal):
         new_time = cls()
         new_time.ordinal = int(ordinal)
         return new_time
@@ -92,12 +119,13 @@ class SubRipTime(object):
 
 
 class SubRipItem(object):
-    RE_ITEM = ur'''(?P<id>\d+)
+
+    RE_ITEM = r'''(?P<sub_id>\d+)
 (?P<start>\d{2}:\d{2}:\d{2},\d{3}) --> (?P<end>\d{2}:\d{2}:\d{2},\d{3})
 (?P<sub_title>.*)'''
 
-    def __init__(self, id=0, start=None, end=None, sub_title=''):
-        self. id = int(id)
+    def __init__(self, sub_id=0, start=None, end=None, sub_title=''):
+        self.id = int(sub_id)
         self.start = start or SubRipTime()
         self.end = end or SubRipTime()
         self.sub_title = unicode(sub_title)
@@ -107,8 +135,10 @@ class SubRipItem(object):
             self.start, self.end, self.sub_title)
 
     def shift(self, *args, **kwargs):
-        """shift(hours, minutes, seconds, micro_seconds)
+        """
+        shift(hours, minutes, seconds, micro_seconds)
 
+        Add given values to start and end attributes.
         All arguments are optional and have a default value of 0.
         """
         self.start.shift(*args, **kwargs)
@@ -122,42 +152,69 @@ class SubRipItem(object):
         datas = dict(match.groupdict())
         datas['start'] = SubRipTime.from_string(datas['start'])
         datas['end'] = SubRipTime.from_string(datas['end'])
-        return cls(**str_keys(datas))
+        return cls(**datas)
 
 
 class SubRipFile(object, UserList):
+    """
+    SubRip file descriptor.
 
-    def __init__(self, source_path='', encoding='utf-8'):
-        """SubRipFile([source_path, [encoding]])
+    Provide a pure Python mapping on all metadatas.
+    """
+
+    def __init__(self, path='', encoding='utf-8'):
+        """
+        SubRipFile([path, [encoding]])
+
+        Encoding is set to utf-8 as default.
         """
         UserList.__init__(self)
+        self.encoding = encoding
+        self.path = path
+
         if path:
-            if path.exists(source_path) and path.isfile(source_path):
-                f = open(source_path, 'rU')
+            if exists(path) and isfile(path):
+                source_file = open(path, 'rU')
             else:
-                f = StringIO(source_path)
+                source_file = StringIO(path)
             string_buffer = StringIO()
-            for line in f:
+            for line in chain(source_file, '\n'):
                 if line.strip():
                     string_buffer.write(line)
                 else:
                     string_buffer.seek(0)
-                    source = unicode(string_buffer.read(), encoding)
+                    source = unicode(string_buffer.read(), self.encoding)
                     new_item = SubRipItem.from_string(source)
                     self.data.append(new_item)
                     string_buffer.truncate(0)
-            f.close()
+            source_file.close()
 
     def shift(self, *args, **kwargs):
-        """shift(hours, minutes, seconds, micro_seconds)
+        """
+        shift(hours, minutes, seconds, micro_seconds)
 
+        Add given values to start and end attributes of each items of file
+        with given values.
         All arguments are optional and have a default value of 0.
         """
         for item in self:
             item.shift(*args, **kwargs)
 
-    def save(self, path, encoding='utf-8'):
-        f = open(path, 'w+')
+    def save(self, path=None, encoding=None):
+        """
+        save([save_path[, encoding]])
+
+        Use init path if no other provided.
+        Use init encoding if no other provided.
+        """
+        self.path = path or self.path
+        self.encoding = encoding or self.encoding
+
+        save_file = open(self.path, 'w+')
         for item in self:
-            f.write(unicode(item).encode(encoding))
-        f.close()
+            save_file.write(unicode(item).encode(self.encoding))
+        save_file.close()
+
+if __name__ == "__main__":
+    import doctest
+    doctest.testmod()
